@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,7 +23,24 @@
 Test case for module FileUtilities
 """
 
+# standard library
+import contextlib
+import ctypes
+import json
+import locale
+import os
+import random
+import re
+import sqlite3
+import stat
+import subprocess
+import sys
+import tempfile
+import time
+import unittest
+import warnings
 
+# local import
 from bleachbit.FileUtilities import (
     bytes_to_human,
     children_in_directory,
@@ -32,12 +49,14 @@ from bleachbit.FileUtilities import (
     delete,
     detect_encoding,
     ego_owner,
-    exists_in_path,
     exe_exists,
+    execute_sqlite3,
+    exists_in_path,
     expand_glob_join,
-    extended_path,
     extended_path_undo,
+    extended_path,
     free_space,
+    get_filesystem_type,
     getsize,
     getsizedir,
     globex,
@@ -48,6 +67,7 @@ from bleachbit.FileUtilities import (
     open_files_lsof,
     OpenFiles,
     same_partition,
+    sync,
     uris_to_paths,
     vacuum_sqlite3,
     whitelisted,
@@ -55,61 +75,67 @@ from bleachbit.FileUtilities import (
     wipe_name,
     wipe_path
 )
-from bleachbit.General import run_external
+from bleachbit.General import gc_collect, run_external
 from bleachbit.Options import options
 from bleachbit import logger
 from tests import common
 
-import json
-import locale
-import os
-import subprocess
-import sys
-import tempfile
-import time
-import unittest
+if 'nt' == os.name:
+    # pylint: disable=import-error
+    import win32api
+    import win32com.shell
+    import win32con
 
 
 def test_ini_helper(self, execute):
     """Used to test .ini cleaning in TestAction and in TestFileUtilities"""
 
-    teststrings = [
-        b'#Test\n[RecentsMRL]\nlist=C:\\Users\\me\\Videos\\movie.mpg,C:\\Users\\me\\movie2.mpg\n\n',
-    ]
-    for teststr in teststrings:
+    teststr = '#Test\n[RecentsMRL]\nlist=C:\\Users\\me\\Videos\\movie.mpg,C:\\Users\\me\\movie2.mpg\n'
+    for encoding in ['utf-8', 'utf-8-sig']:
+        with self.subTest(encoding=encoding):
+            extra_size = 0
+            if 'utf-8-sig' == encoding:
+                extra_size = 3
+            if 'nt' == os.name:
+                extra_size += teststr.count('\n')
 
-        # create test file
-        filename = self.write_file('bleachbit-test-ini', teststr)
-        self.assertExists(filename)
-        size = os.path.getsize(filename)
-        self.assertEqual(len(teststr), size)
+            # create test file
+            filename = self.write_file(
+                'bleachbit-test-ini', teststr, mode='w', encoding=encoding)
+            self.assertExists(filename)
+            size = os.path.getsize(filename)
+            self.assertEqual(len(teststr) + extra_size, size)
 
-        # section does not exist
-        execute(filename, 'Recents', None)
-        self.assertEqual(len(teststr), os.path.getsize(filename))
+            # The section does not exist, so no change.
+            execute(filename, 'Recents', None)
+            self.assertEqual(len(teststr) + extra_size,
+                             os.path.getsize(filename))
 
-        # parameter does not exist
-        execute(filename, 'RecentsMRL', 'files')
-        self.assertEqual(len(teststr), os.path.getsize(filename))
+            # The parameter does not exist, so no change.
+            execute(filename, 'RecentsMRL', 'files')
+            self.assertEqual(len(teststr) + extra_size,
+                             os.path.getsize(filename))
 
-        # parameter does exist
-        execute(filename, 'RecentsMRL', 'list')
-        self.assertEqual(14, os.path.getsize(filename))
+            # The parameter does exist, so the file shrinks.
+            # The file will be size 14 if chardet is available.
+            # Otherwise, size will be 17 with BOM.
+            execute(filename, 'RecentsMRL', 'list')
+            self.assertIn(os.path.getsize(filename), (14, 17))
 
-        # section does exist
-        execute(filename, 'RecentsMRL', None)
-        self.assertEqual(0, os.path.getsize(filename))
+            # The section does exist, so the file shrinks.
+            execute(filename, 'RecentsMRL', None)
+            self.assertEqual(0, os.path.getsize(filename))
 
-        # clean up
-        delete(filename)
-        self.assertNotExists(filename)
+            # clean up
+            delete(filename)
+            self.assertNotExists(filename)
 
 
 def test_json_helper(self, execute):
     """Used to test JSON cleaning in TestAction and in TestFileUtilities"""
 
     def load_js(js_fn):
-        with open(js_fn, 'r') as js_fd:
+        with open(js_fn, 'r', encoding='utf-8') as js_fd:
             return json.load(js_fd)
 
     expected = {'deleteme': 1, 'spareme': {'deletemetoo': 1}}
@@ -150,14 +176,45 @@ def test_json_helper(self, execute):
 class FileUtilitiesTestCase(common.BleachbitTestCase):
     """Test case for module FileUtilities"""
 
-    def test_bytes_to_human(self):
-        """Unit test for class bytes_to_human"""
-
-        old_locale = locale.getlocale(locale.LC_NUMERIC)
+    def setUp(self):
+        """Call before each test method"""
+        super().setUp()
+        self.old_locale = locale.getlocale(locale.LC_NUMERIC)
         locale.setlocale(locale.LC_NUMERIC, 'C')
 
-        # test one-way conversion for predefined values
-        # each test is a tuple in the format: (bytes, SI, EIC)
+    def tearDown(self):
+        """Call after each test method"""
+        if self.old_locale == (None, None):
+            locale.setlocale(locale.LC_NUMERIC, 'C')
+            return
+        try:
+            if self.old_locale[0] is None:
+                locale.setlocale(locale.LC_NUMERIC, None)
+            else:
+                try:
+                    # First try with the full locale string
+                    locale.setlocale(locale.LC_NUMERIC, '.'.join(
+                        filter(None, self.old_locale)))
+                except locale.Error:
+                    # If that fails, try just the language code part
+                    try:
+                        locale.setlocale(locale.LC_NUMERIC, self.old_locale[0])
+                    except locale.Error as e:
+                        print(
+                            "Failed to restore locale with just language code "
+                            f"{self.old_locale[0]}: {e}")
+        except locale.Error as e:
+            print(f"Failed to restore locale {self.old_locale}: {e}")
+        # Check that running getlocale again does not raise an exception.
+        # For me, getlocale() fails after successful setlocale(..., 'en_MX') on Windows.
+        locale.getlocale(locale.LC_NUMERIC)
+        self.assertEqual(locale.getlocale(locale.LC_NUMERIC), self.old_locale)
+
+    def test_bytes_to_human_one_way(self):
+        """Test one-way conversion of bytes_to_human()"""
+
+        # Test one-way conversion for predefined values
+        # Each test is a tuple in the format: (bytes, SI, EIC)
         tests = ((-1, '-1B', '-1B'),
                  (0, '0B', '0B'),
                  (1, '1B', '1B'),
@@ -179,35 +236,41 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         for test in tests:
             iec = bytes_to_human(test[0])
             self.assertEqual(test[2], iec,
-                             'bytes_to_human(%d) IEC = %s but expected %s' % (test[0], iec, test[2]))
+                             f"bytes_to_human({test[0]}) IEC = {iec}"
+                             f" but expected {test[2]}")
 
         options.set('units_iec', False)
         for test in tests:
             si = bytes_to_human(test[0])
             self.assertEqual(test[1], si,
-                             'bytes_to_human(%d) SI = %s but expected %s' % (test[0], si, test[1]))
+                             f"bytes_to_human({test[0]}) SI = {si}"
+                             f" but expected {test[1]}")
+
+    def test_bytes_to_human_roundtrip(self):
+        """Test roundtrip conversion of bytes_to_human()
+
+        Example: 1,964,950 -> 2MB -> 2,000,000 with difference of 1.78% (0.0178).
+        """
 
-        # test roundtrip conversion for random values
-        import random
         for _n in range(0, 1000):
             bytes1 = random.randrange(0, 1000 ** 4)
             human = bytes_to_human(bytes1)
             bytes2 = human_to_bytes(human)
             error = abs(float(bytes2 - bytes1) / bytes1)
-            self.assertLess(abs(error), 0.01, "%d (%s) is %.2f%% different than %d" %
-                            (bytes1, human, error * 100, bytes2))
+            self.assertLess(abs(
+                error), 0.02, f"{bytes1:,} ({human}) is "
+                f"{error * 100:.2f}% different than {bytes2:,}")
 
-        # test localization
-        if hasattr(locale, 'format_string'):
-            try:
-                locale.setlocale(locale.LC_NUMERIC, 'de_DE.utf8')
-            except:
-                logger.warning('exception when setlocale to de_DE.utf8')
-            else:
-                self.assertEqual("1,01GB", bytes_to_human(1000 ** 3 + 5812389))
-
-        # clean up
-        locale.setlocale(locale.LC_NUMERIC, old_locale)
+    def test_bytes_to_human_localization(self):
+        """Test localization of bytes_to_human()"""
+        if not hasattr(locale, 'format_string'):
+            self.skipTest('Locale module does not support format_string')
+        try:
+            locale.setlocale(locale.LC_NUMERIC, 'de_DE.utf8')
+        except locale.Error as e:
+            logger.warning('exception when setlocale to de_DE.utf8: %s', e)
+        else:
+            self.assertEqual("1,01GB", bytes_to_human(1000 ** 3 + 5812389))
 
     def test_children_in_directory(self):
         """Unit test for function children_in_directory()"""
@@ -271,25 +334,6 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         delete('does-not-exist', ignore_missing=True)
         self.assertRaises(OSError, delete, 'does-not-exist')
 
-    def test_delete_not_empty(self):
-        """Test for scenario directory is not empty"""
-        dirname = os.path.join(self.tempdir, 'a_dir')
-        os.mkdir(dirname)
-        self.assertTrue(is_dir_empty(dirname))
-        fn = os.path.join(dirname, 'a_file')
-        common.touch_file(fn)
-        self.assertFalse(is_dir_empty(dirname))
-        self.assertExists(fn)
-        self.assertExists(dirname)
-        self.assertExists(self.tempdir)
-
-        # Make sure shredding does not leave a renamed directory like
-        # in https://github.com/bleachbit/bleachbit/issues/783
-        for allow_shred in (False, True):
-            delete(dirname, allow_shred=allow_shred)
-            self.assertExists(fn)
-            self.assertExists(dirname)
-
     def delete_helper(self, shred):
         """Called by test_delete() with shred = False and = True"""
 
@@ -330,8 +374,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
         def symlink_helper(link_fn):
             if 'nt' == os.name:
-                from win32com.shell import shell
-                if not shell.IsUserAnAdmin():
+                # pylint: disable=no-member
+                if not win32com.shell.shell.IsUserAnAdmin():
                     self.skipTest(
                         'skipping symlink test because of insufficient privileges')
 
@@ -376,18 +420,17 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
         if 'nt' == os.name:
             logger.debug('testing symbolic link')
-            import ctypes
             kern = ctypes.windll.LoadLibrary("kernel32.dll")
 
             def win_symlink(src, linkname):
                 rc = kern.CreateSymbolicLinkW(linkname, src, 0)
                 if rc == 0:
-                    print('CreateSymbolicLinkW(%s, %s)' % (linkname, src))
-                    print('CreateSymolicLinkW() failed, error = %s' %
-                          ctypes.FormatError())
+                    print(f'CreateSymbolicLinkW({linkname}, {src})')
+                    print(
+                        f'CreateSymolicLinkW() failed, error = {ctypes.FormatError()}')
                     self.assertNotEqual(rc, 0)
             symlink_helper(win_symlink)
-            
+
             return
 
         # below this point, only posix
@@ -415,25 +458,13 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         delete(path, shred)
         self.assertNotExists(path)
 
-    def test_delete_read_only(self):
-        """Unit test for delete() with read-only file"""
-        for shred in (False, True):
-            fn = os.path.join(self.tempdir, 'read-only')
-            common.touch_file(fn)
-            import stat
-            os.chmod(fn, stat.S_IREAD)
-            self.assertExists(fn)
-            delete(fn, shred=shred)
-            self.assertNotExists(fn)
-
     @common.skipUnlessWindows
     def test_delete_hidden(self):
         """Unit test for delete() with hidden file"""
         for shred in (False, True):
             fn = os.path.join(self.tempdir, 'hidden')
             common.touch_file(fn)
-            import win32api
-            import win32con
+            # pylint: disable=possibly-used-before-assignment, c-extension-no-member
             win32api.SetFileAttributes(fn, win32con.FILE_ATTRIBUTE_HIDDEN)
             self.assertExists(fn)
             delete(fn, shred=shred)
@@ -444,7 +475,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         """Unit test for delete() with locked file"""
         # set up
         def test_delete_locked_setup():
-            (fd, filename) = tempfile.mkstemp(prefix='bleachbit-test-worker')
+            (fd, filename) = tempfile.mkstemp(
+                prefix='bleachbit-test-fileutilities')
             os.write(fd, b'123')
             os.close(fd)
             self.assertExists(filename)
@@ -458,6 +490,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         f = os.open(filename, os.O_WRONLY | os.O_EXCL)
         self.assertExists(filename)
         self.assertEqual(3, getsize(filename))
+        # pylint: disable=undefined-variable
         with self.assertRaises(WindowsError):
             delete(filename)
         os.close(f)
@@ -474,6 +507,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             fd = os.open(filename, os.O_APPEND | os.O_EXCL)
             self.assertExists(filename)
             self.assertEqual(3, getsize(filename))
+            # pylint: disable=undefined-variable
             with self.assertRaises(WindowsError):
                 delete(filename, shred=allow_shred, allow_shred=allow_shred)
             os.close(fd)
@@ -494,41 +528,97 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         os.mkdir(from_dir)
         os.mkdir(to_dir)
         args = ['mount', '--bind', from_dir, to_dir]
-        (rc, stdout, stderr) = run_external(args)
-        msg = 'error calling mount\nargs=%s\nstderr=%s' % (args, stderr)
-        self.assertEqual(rc, 0, msg)
+        (rc, _, stderr) = run_external(args)
+        self.assertEqual(
+            rc, 0, f'error calling mount\nargs={args}\nstderr={stderr}')
 
         delete(to_dir)
 
         args = ['umount', to_dir]
-        (rc, stdout, stderr) = run_external(args)
-        msg = 'error calling umount\nargs=%s\nstderr=%s' % (args, stderr)
-        self.assertEqual(rc, 0, msg)
+        (rc, _, stderr) = run_external(args)
+        self.assertEqual(
+            rc, 0, f'error calling umount\nargs={args}\nstderr={stderr}')
+
+    def test_delete_not_empty(self):
+        """Test for scenario directory is not empty"""
+        # common.py puts bleachbit.ini in self.tempdir, but it may
+        # not be flushed
+        dirname = os.path.join(self.tempdir, 'a_dir')
+        os.mkdir(dirname)
+        self.assertFalse(is_dir_empty(self.tempdir))
+        self.assertTrue(is_dir_empty(dirname))
+        fn = os.path.join(dirname, 'a_file')
+        common.touch_file(fn)
+        self.assertFalse(is_dir_empty(dirname))
+        self.assertExists(fn)
+        self.assertExists(dirname)
+        self.assertExists(self.tempdir)
+
+        # Make sure shredding does not leave a renamed directory like
+        # in https://github.com/bleachbit/bleachbit/issues/783
+        for allow_shred in (False, True):
+            delete(dirname, allow_shred=allow_shred)
+            self.assertExists(fn)
+            self.assertExists(dirname)
+        os.remove(fn)
+        self.assertTrue(is_dir_empty(dirname))
+
+    def test_delete_read_only(self):
+        """Unit test for delete() with read-only file"""
+        for shred in (False, True):
+            fn = os.path.join(self.tempdir, 'read-only')
+            common.touch_file(fn)
+            os.chmod(fn, stat.S_IREAD)
+            self.assertExists(fn)
+            delete(fn, shred=shred)
+            self.assertNotExists(fn)
 
     def test_detect_encoding(self):
         """Unit test for detect_encoding"""
         eat_glass = '나는 유리를 먹을 수 있어요. 그래도 아프지 않아요'
+        bom = '\ufeff' + eat_glass  # Add BOM for utf-8-sig
         tests = (('This is just an ASCII file', 'ascii'),
                  (eat_glass, 'utf-8'),
-                 (eat_glass, 'EUC-KR'))
+                 (eat_glass, 'EUC-KR'),
+                 (bom, 'UTF-8-SIG'))
         for file_contents, expected_encoding in tests:
-            with tempfile.NamedTemporaryFile(mode='w', delete=False, encoding=expected_encoding) as temp:
-                temp.write(file_contents)
-                temp.flush()
-            det = detect_encoding(temp.name)
-            self.assertEqual(det, expected_encoding)
+            with self.subTest(encoding=expected_encoding):
+                with tempfile.NamedTemporaryFile(mode='w', delete=False,
+                                                 encoding=expected_encoding) as temp:
+                    temp.write(file_contents)
+                    temp.flush()
+                det = detect_encoding(temp.name)
+                self.assertEqual(
+                    det, expected_encoding,
+                    f"{file_contents} -> {det}, check that chardet is available")
 
     @common.skipIfWindows
     def test_ego_owner(self):
         """Unit test for ego_owner()"""
+        # pylint: disable=no-member
         self.assertEqual(ego_owner('/bin/ls'), os.getuid() == 0)
 
-    def test_exists_in_path(self):
-        """Unit test for exists_in_path()"""
-        filename = 'ls'
-        if 'nt' == os.name:
-            filename = 'cmd.exe'
-        self.assertTrue(exists_in_path(filename))
+    def test_execute_sqlite3(self):
+        """Unit test for execute_sqlite3()"""
+        db_path = self.mkstemp(prefix='bleachbit-test-file', suffix='.sqlite')
+
+        cmds = ['CREATE TABLE test (id INTEGER PRIMARY KEY, name TEXT)',
+                "INSERT INTO test (name) VALUES ('A'), ('B')",
+                "UPDATE test SET name = 'C' WHERE name = 'B'",
+                "DELETE FROM test WHERE name = 'C'"]
+
+        execute_sqlite3(db_path, ';'.join(cmds))
+        execute_sqlite3(db_path, 'vacuum')
+
+        with contextlib.closing(sqlite3.connect(db_path)) as conn:
+            res = conn.execute('select 1 from test where name = "A"')
+            row = res.fetchone()
+            self.assertIsNotNone(row)
+            self.assertEqual(row[0], 1)
+
+        gc_collect()
+        os.unlink(db_path)
+        self.assertNotExists(db_path)
 
     def test_exe_exists(self):
         """Unit test for exe_exists()"""
@@ -541,8 +631,29 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                      ('cmd.exe', True),
                      ('doesnotexist', False),
                      ('c:\\windows\\doesnotexist.exe', False)]
-        for test in tests:
-            self.assertEqual(exe_exists(test[0]), test[1])
+        for exe, expected in tests:
+            with self.subTest(exe=exe, expected=expected):
+                if os.name == 'posix' and not os.getenv('PATH') and not os.path.isabs(exe):
+                    self.skipTest('PATH not set')
+                self.assertEqual(exe_exists(exe), expected,
+                                 f"{exe} -> {expected}")
+
+    def test_exists_in_path(self):
+        """Unit test for exists_in_path()"""
+        filename = 'ls'
+        if 'nt' == os.name:
+            filename = 'cmd.exe'
+        if 'posix' == os.name and not os.getenv('PATH'):
+            self.assertFalse(exists_in_path(filename))
+        else:
+            self.assertTrue(exists_in_path(filename))
+        self.assertFalse(exists_in_path('doesnotexist'))
+        if 'posix' == os.name:
+            with self.assertRaises(AssertionError):
+                exists_in_path('/usr/bin/doesnotexist')
+        if 'nt' == os.name:
+            with self.assertRaises(AssertionError):
+                exists_in_path('c:\\does\\not\\exist.exe')
 
     def test_expand_glob_join(self):
         """Unit test for expand_glob_join()"""
@@ -551,10 +662,44 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         if 'nt' == os.name:
             expand_glob_join(r'c:\windows', '*.exe')
 
-    def test_expandvars(self):
-        """Unit test for expandvars()."""
-        expanded = os.path.expandvars('$HOME')
-        self.assertIsString(expanded)
+    def test_expand_vars_user(self):
+        """Unit test for expandvars() and expanduser()
+
+        We had custom implementations of expandvars() and expanduser()
+        until commit 3a3913 that switched to Python 3.
+        """
+
+        if 'nt' == os.name:
+            home_vars = ('%USERPROFILE%', '$userprofile', '${USERprofile}')
+        else:
+            home_vars = ['$HOME', '${HOME}']
+        for var in home_vars:
+            if not os.getenv(var):
+                self.skipTest(f'Environment variable {var} not set')
+            expanded = os.path.expandvars(var)
+            self.assertIsString(expanded)
+            self.assertNotEqual(
+                expanded, var, 'Environment variable was not expanded')
+            self.assertEqual(expanded, os.path.expanduser('~'))
+            self.assertExists(expanded)
+            # An absolute path should not be altered.
+            self.assertEqual(expanded, os.path.expandvars(expanded))
+            self.assertEqual(expanded, os.path.expanduser(expanded))
+
+    def test_expand_vars_no_change(self):
+        """Unit test for expandvars() and expanduser()
+
+        These expect no change.
+        """
+        # An empty string should not be altered.
+        self.assertEqual(os.path.expandvars(''), '')
+
+        # Non-existent variables should not be altered.
+        self.assertEqual(os.path.expandvars('$nonexistent'), '$nonexistent')
+
+        # A relative path should not be expanded.
+        for p in ('common', 'common/', 'common', 'home', 'userprofile'):
+            self.assertEqual(os.path.expanduser(p), p)
 
     def test_extended_path(self):
         """Unit test for extended_path() and extended_path_undo()"""
@@ -584,24 +729,73 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         self.assertGreater(result, -1)
         self.assertIsInteger(result)
 
-        # compare to WMIC
-        if 'nt' != os.name:
-            return
-        args = ['wmic',  'LogicalDisk', 'get', 'DeviceID,', 'FreeSpace']
-        (rc, stdout, stderr) = run_external(args)
-        if rc:
-            print('error calling WMIC\nargs=%s\nstderr=%s' % (args, stderr))
-            return
-        import re
-        for line in stdout.split('\n'):
-            line = line.strip()
-            if not re.match(r'([A-Z]):\s+(\d+)', line):
-                continue
-            drive, bytes_free = re.split(r'\s+', line)
-            print('Checking free space for %s' % drive)
-            bytes_free = int(bytes_free)
-            free = free_space(drive)
-            self.assertEqual(bytes_free, free)
+    @common.skipUnlessWindows
+    def test_free_space_windows(self):
+        """Unit test for free_space() on Windows
+
+        Repeat because of possible race condition.
+        """
+        args = ['wmic', 'LogicalDisk', 'get', 'DeviceID,', 'FreeSpace']
+        max_attempts = 3
+
+        def compare_free_space():
+            """Returns whether all drives have equal free space"""
+            (rc, stdout, stderr) = run_external(args)
+            self.assertEqual(
+                rc, 0, f'error calling WMIC\nargs={args}\nstderr={stderr}')
+            lines = stdout.splitlines()
+            self.assertGreater(len(lines), 0)
+            for line in lines:
+                line = line.strip()
+                if not re.match(r'([A-Z]):\s+(\d+)', line):
+                    continue
+                drive, bytes_free = re.split(r'\s+', line)
+                bytes_free = int(bytes_free)
+                free = free_space(drive)
+                if free != bytes_free:
+                    logger.debug(
+                        'Free space mismatch for drive %s: %s != %s', drive, free, bytes_free)
+                    return False
+            return True
+
+        for attempt in range(max_attempts):
+            if attempt > 0:
+                time.sleep(0.5)
+            if compare_free_space():
+                return
+            if attempt == max_attempts - 1:
+                self.fail(
+                    f'Failed to find equal free space after {max_attempts} attempts')
+
+    def test_get_filesystem_type(self):
+        """Unit test for get_filesystem_type()"""
+        home = os.path.expanduser('~')
+        if os.name == 'nt':
+            path = self.tempdir
+            fs_type = get_filesystem_type(path)[0]
+            while path:
+                print(path)
+                self.assertEqual(get_filesystem_type(path)[0], fs_type)
+                self.assertEqual(get_filesystem_type(path.lower())[0], fs_type)
+                self.assertEqual(get_filesystem_type(path.upper())[0], fs_type)
+                if path == os.path.dirname(path):
+                    break
+                path = os.path.dirname(path)
+            self.assertNotEqual(fs_type, 'unknown')
+            self.assertEqual(get_filesystem_type(r'a:\\')[0], 'unknown')
+            self.assertEqual(get_filesystem_type(r'Z:\\')[0], 'unknown')
+            for check_path in (home, r'c:\\', r'c:'):
+                self.assertEqual(get_filesystem_type(
+                    check_path)[0], 'NTFS', check_path)
+                self.assertEqual(get_filesystem_type(
+                    check_path.lower())[0], 'NTFS')
+                self.assertEqual(get_filesystem_type(
+                    check_path.upper())[0], 'NTFS')
+        elif os.name == 'posix':
+            for check_path in (home, '/'):
+                detected_fs = get_filesystem_type(check_path)[0]
+                self.assertIn(detected_fs, ['btrfs', 'ext4', 'ext3', 'squashfs', 'unknown'],
+                              f"Unexpected file system type for {check_path}: {detected_fs}")
 
     def test_getsize(self):
         """Unit test for method getsize()"""
@@ -616,7 +810,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                 # Expand the directory names, which are in the short format,
                 # to test the case where the full path (including the directory)
                 # is longer than 255 characters.
-                import win32api
+                # pylint: disable=possibly-used-before-assignment, c-extension-no-member
                 lname = win32api.GetLongPathNameW(extended_path(filename))
                 self.assertEqual(getsize(lname), 10 * 12345)
                 # this function returns a byte string instead of Unicode
@@ -628,10 +822,11 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             if 'posix' == os.name:
                 encoding = sys.getdefaultencoding()
                 output = str(subprocess.Popen(
-                    ["du", "-h", filename], stdout=subprocess.PIPE).communicate()[0], encoding=encoding)
+                    ["du", "-h", filename],
+                    stdout=subprocess.PIPE).communicate()[0], encoding=encoding)
                 output = output.replace("\n", "")
-                du_size = output.split("\t")[0] + "B"
-                print("output = '%s', size='%s'" % (output, du_size))
+                du_size = output.split('\t', maxsplit=1)[0] + "B"
+                print(f"output = '{output}', size='{du_size}'")
                 du_bytes = human_to_bytes(du_size, 'du')
                 print(output, du_size, du_bytes)
                 self.assertEqual(getsize(filename), du_bytes)
@@ -665,7 +860,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             delete(linkname)
         os.symlink(filename, linkname)
         self.assertLess(getsize(linkname), 8192,
-                        "Symlink size is %d" % getsize(filename))
+                        f"Symlink size is {getsize(filename)}")
         delete(filename)
 
         if 'darwin' == sys.platform:
@@ -695,7 +890,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         """Unit test for guess_overwrite_paths()"""
         for path in guess_overwrite_paths():
             self.assertTrue(os.path.isdir(path),
-                            '%s is not a directory' % path)
+                            f'{path} is not a directory')
 
     def test_human_to_bytes(self):
         """Unit test for human_to_bytes()"""
@@ -707,13 +902,13 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         valid = {'1kB': 1000,
                  '1.1MB': 1100000,
                  '12B': 12,
-                 '1.0M': 1000*1000,
+                 '1.0M': 1000 * 1000,
                  '1TB': 1000**4,
                  '1000': 1000}
         for test, result in valid.items():
             self.assertEqual(human_to_bytes(test), result)
 
-        self.assertEqual(human_to_bytes('1 MB', 'du'), 1024*1024)
+        self.assertEqual(human_to_bytes('1 MB', 'du'), 1024 * 1024)
 
     def test_listdir(self):
         """Unit test for listdir()"""
@@ -723,6 +918,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         elif 'nt' == os.name:
             dir1 = os.path.expandvars(r'%windir%\fonts')
             dir2 = os.path.expandvars(r'%windir%\logs')
+        else:
+            raise NotImplementedError()
         # If these directories do not exist, the test results are not valid.
         self.assertExists(dir1)
         self.assertExists(dir2)
@@ -743,6 +940,33 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         for pathname in paths12:
             self.assertLExists(pathname)
 
+    def test_open_files_lsof(self):
+        """Unit test for open_files_lsof()"""
+        self.assertEqual(list(open_files_lsof(
+            lambda: 'n/bar/foo\nn/foo/bar\nnoise')), ['/bar/foo', '/foo/bar'])
+
+    @common.skipIfWindows
+    def test_open_files(self):
+        """Unit test for class OpenFiles"""
+
+        filename = os.path.join(self.tempdir, 'bleachbit-test-open-files')
+        with open(filename, 'wb'):
+            openfiles = OpenFiles()
+            ago = None
+            if openfiles.last_scan_time:
+                ago = time.time() - openfiles.last_scan_time
+            self.assertTrue(openfiles.is_open(filename),
+                            f"Expected is_open({filename}) to return True)\n"
+                            f"openfiles.last_scan_time (ago)={ago}\n"
+                            f"openfiles.files={openfiles.files}")
+
+        openfiles.scan()
+        self.assertFalse(openfiles.is_open(filename))
+
+        os.unlink(filename)
+        openfiles.scan()
+        self.assertFalse(openfiles.is_open(filename))
+
     def test_same_partition(self):
         """Unit test for same_partition()"""
         home = os.path.expanduser('~')
@@ -751,12 +975,17 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             self.assertFalse(same_partition(home, '/dev'))
         elif 'nt' == os.name:
             home_drive = os.path.splitdrive(home)[0]
+            # pylint: disable=import-outside-toplevel
             from bleachbit.Windows import get_fixed_drives
             for drive in get_fixed_drives():
                 this_drive = os.path.splitdrive(drive)[0]
                 self.assertEqual(same_partition(home, drive),
                                  home_drive == this_drive)
 
+    def test_sync(self):
+        """Unit test for sync()"""
+        sync()
+
     def test_uris_to_paths(self):
         """Unit test for uris_to_paths()"""
         self.assertEqual(uris_to_paths(['']), [])
@@ -778,6 +1007,42 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         uri_s = ['foo://bar']
         self.assertEqual(uris_to_paths(uri_u + uri_w + uri_s), path_u + path_w)
 
+    def test_vacuum_sqlite3(self):
+        """Unit test for method vacuum_sqlite3()"""
+        path = os.path.join(self.tempdir, 'bleachbit.tmp.sqlite3')
+        conn = sqlite3.connect(path)
+        conn.execute('create table numbers (number)')
+        conn.commit()
+        empty_size = getsize(path)
+
+        def number_generator():
+            for x in range(1, 10000):
+                yield (x, )
+        conn.executemany(
+            'insert into numbers (number) values ( ? ) ', number_generator())
+        conn.commit()
+        self.assertLess(empty_size, getsize(path))
+        conn.execute('delete from numbers')
+        conn.commit()
+        conn.close()
+
+        vacuum_sqlite3(path)
+        self.assertEqual(empty_size, getsize(path))
+
+        gc_collect()
+        delete(path)
+        self.assertNotExists(path)
+
+    def test_sqlite_loop(self):
+        """Repeat SQLite tests
+
+        This may raise ResourceWarning on Python 3.13.
+        """
+        warnings.simplefilter("error")
+        for _ in range(50):
+            self.test_execute_sqlite3()
+            self.test_vacuum_sqlite3()
+
     def test_whitelisted(self):
         """Unit test for whitelisted()"""
         # setup
@@ -883,8 +1148,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             for p in paths:
                 _ = whitelisted(p)
         t1 = time.time()
-        logger.info('whitelisted() with {} repetitions and {} paths took {:.3g} seconds '.format(
-            reps, len(paths), t1 - t0))
+        logger.info('whitelisted() with %d repetitions and %d paths took %g seconds',
+                    reps, len(paths), t1 - t0)
 
         options.set_whitelist_paths(old_whitelist)
 
@@ -966,7 +1231,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         self.assertNotExists(dir0)
 
     @unittest.skipUnless(os.getenv('ALLTESTS') is not None,
-                         'warning: skipping long test test_wipe_path() because environment variable ALLTESTS not set')
+                         'warning: skipping long test test_wipe_path() because'
+                         ' environment variable ALLTESTS not set')
     def test_wipe_path(self):
         """Unit test for wipe_path()"""
 
@@ -975,60 +1241,15 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             pass
 
     def test_wipe_path_fast(self):
-        next(wipe_path(self.tempdir, True))
+        """Unit test for wipe_path() with fast mode
 
-    def test_vacuum_sqlite3(self):
-        """Unit test for method vacuum_sqlite3()"""
-        import sqlite3
-
-        path = os.path.abspath('bleachbit.tmp.sqlite3')
-        if os.path.lexists(path):
-            delete(path)
-
-        conn = sqlite3.connect(path)
-        conn.execute('create table numbers (number)')
-        conn.commit()
-        empty_size = getsize(path)
-
-        def number_generator():
-            for x in range(1, 10000):
-                yield (x, )
-        conn.executemany(
-            'insert into numbers (number) values ( ? ) ', number_generator())
-        conn.commit()
-        self.assertLess(empty_size, getsize(path))
-        conn.execute('delete from numbers')
-        conn.commit()
-        conn.close()
-
-        vacuum_sqlite3(path)
-        self.assertEqual(empty_size, getsize(path))
-
-        delete(path)
-
-    @common.skipIfWindows
-    def test_OpenFiles(self):
-        """Unit test for class OpenFiles"""
-
-        filename = os.path.join(self.tempdir, 'bleachbit-test-open-files')
-        f = open(filename, 'w')
-        openfiles = OpenFiles()
-        self.assertTrue(openfiles.is_open(filename),
-                        "Expected is_open(%s) to return True)\n"
-                        "openfiles.last_scan_time (ago)=%s\n"
-                        "openfiles.files=%s" %
-                        (filename,
-                         time.time() - openfiles.last_scan_time,
-                         openfiles.files))
-
-        f.close()
-        openfiles.scan()
-        self.assertFalse(openfiles.is_open(filename))
-
-        os.unlink(filename)
-        openfiles.scan()
-        self.assertFalse(openfiles.is_open(filename))
-
-    def test_open_files_lsof(self):
-        self.assertEqual(list(open_files_lsof(
-            lambda: 'n/bar/foo\nn/foo/bar\nnoise')), ['/bar/foo', '/foo/bar'])
+        This test runs three iterations of the generator
+        and then aborts. Each iteration takes a little more
+        than two seconds.
+        """
+        counter = 0
+        for _i in wipe_path(self.tempdir, True):
+            counter += 1
+            if counter >= 3:
+                break
+        self.assertGreater(counter, 0)
